//****************************************************************************
// File: mainwnd.c
//
// Purpose: Main overlapped window's message handler
//
// Functions:
//    MainWndProc() - message handler for main overlapped window
//    DoCommands() - called by MainWndProc to handle all WM_COMMAND messages
//
// Development Team:  Matt Leber
//
// Written by Microsoft Product Support Services, Windows Developer Support
// Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include <windows.h>
#include "global.h"
#include "about.h"
#include "mainwnd.h"

// Local function prototypes.

long DoCommands (HWND hWnd, unsigned message, WORD wParam, LONG lParam);

//****************************************************************************
// Function: MainWndProc
//
// Purpose: Message handler for main overlapped window.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: Depends on message.
//
// History:  Date       Author        Reason
//           1/27/92                  Created
//****************************************************************************

long FAR PASCAL MainWndProc (HWND hWnd, unsigned message,
                             WORD wParam, LONG lParam)
{
   switch (message)
   {
      // Dispatch WM_COMMAND messages to our command handler, DoCommands().
      case WM_COMMAND:
         return DoCommands(hWnd, message, wParam, lParam);

      // On WM_DESTROY, terminate this app by posting a WM_QUIT message.

      case WM_DESTROY:
         PostQuitMessage(0);
         break;

      // We didn't handle, pass to DefWindowProc.

      default:
         return DefWindowProc(hWnd, message, wParam, lParam);
   }


   return (NULL);
}

//****************************************************************************
// Function: DoCommands
//
// Purpose: Called by MainWndProc() to handle all WM_COMMAND messages.
//
// Parameters:
//    hWnd    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns : Depends on message.
//
// History:  Date       Author        Reason
//           1/27/92                  Created
//           2/20/92    MSL           Added DLLDialogBox call
//****************************************************************************

long DoCommands (HWND hWnd, unsigned message, WORD wParam, LONG lParam)
{
   switch (wParam)
   {
      // Put up the About box.
      case IDM_ABOUT:
      {
         FARPROC lpDlgProc;

         lpDlgProc = MakeProcInstance(AboutDlg, ghInst);

         DialogBox(ghInst,        // current instance
                   AboutBoxName,  // resource to use 
                   hWnd,          // parent handle   
                   lpDlgProc);    // About() instance address

         FreeProcInstance(lpDlgProc);
         break;
      }

      case IDM_DLLDLG:
         DLLDialogBox(hWnd); //  Call DLL Function which brings up a
                             //  dialog box
         break;

      //  User picked File.Exit, so post a WM_CLOSE to ourselves.  If
      //  WM_CLOSE is passed onto DefWindowProc, DestroyWindow is called.
      //  Posting the WM_CLOSE allows the application to respond to the 
      //  WM_CLOSE and decide whether or not it really wants to be shut
      //  down.  For this application, it doesn't really matter.  In 
      //  general, however, it is a good idea.

      case IDM_EXIT:
         PostMessage(hWnd, WM_CLOSE, 0, 0L);
         break;

      // Must be some system command -- pass it on to the default
      // window procedure.

      default:
         return DefWindowProc(hWnd, message, wParam, lParam);
   }

   return (NULL);
}
