(*******************************************************************)
(*                                                                 *)
(* 		 Microworks ObjectMate 2.52  	           								     *)
(*                                                                 *)
(*     Windows Interface Develpment Kit the Borland Languages.     *)
(*                                                                 *)
(*		 SFXWNDS2.PAS : Edit and File Window Unit               		 *)
(*     Modified for font & color selection & common                *)
(*     dialogs.                                                    *)
(*																													       *)
(*     Copyright 1992-94 Microworks Sydney, Australia.	           *)
(*                                                                 *)
(*******************************************************************)

unit SFXWnds2;

interface

uses WinTypes, WinProcs, Win31, SFX200,
		 {$IFDEF Ver15}
			 WObjects, StdWnds;
		 {$ELSE}
			 Objects, OWindows, ODialogs, OStdWnds, Strings, CommDlg, WinDos;
		 {$ENDIF}

const

	mws_3DFrame  = $1000;
	mws_SFXFrame = $2000;

type

  TFilename = array [0..255] of Char;

	PSFXEditWindow = ^TSFXEditWindow;
	TSFXEditWindow = object(TEditWindow)
    Font : TLogFont;       { Logical font }
    TheFont : HFont;
		constructor Init (AParent: PWindowsObject; ATitle: PChar);
		constructor Load (var S: TStream);
		procedure Store (var S: TStream);
		procedure GetWindowClass (var AWndClass: TWndClass); virtual;
    procedure SetupWindow; virtual;
    procedure MakeDefaultFont(var AFont: HFont);
		procedure WMCtlColor (var Msg: TMessage); virtual wm_First + wm_CtlColor;
		procedure WMSize (var Msg: TMessage); virtual wm_First + wm_Size;
	end;

	PSFXFileWindow = ^TSFXFileWindow;
	TSFXFileWindow = object(TFileWindow)
    Font : TLogFont;       { Logical font }
    TheFont : HFont;
		constructor Init (AParent: PWindowsObject; ATitle, AFileName: PChar);
		constructor Load (var S: TStream);
		procedure Store (var S: TStream);
		procedure GetWindowClass (var AWndClass: TWndClass); virtual;
    procedure SetupWindow; virtual;
    procedure MakeDefaultFont(var AFont: HFont);
    function Save: Boolean;
    function SaveAs: Boolean;
    procedure CMFileSave(var Msg: TMessage);
      virtual cm_First + cm_FileSave;
    procedure CMFileSaveAs(var Msg: TMessage);
      virtual cm_First + cm_FileSaveAs;
		procedure WMCtlColor (var Msg: TMessage); virtual wm_First + wm_CtlColor;
		procedure WMSize (var Msg: TMessage); virtual wm_First + wm_Size;
	 {procedure WMWindowPosChanging (var Msg: TMessage); virtual wm_First + wm_WindowPosChanging;}
	end;

	procedure RegisterSFXWnds;

const
	{ Stream registration records }
	RSFXEditWindow : TStreamRec = (
		ObjType : 15216;
		VmtLink : Ofs(TypeOf(TSFXEditWindow)^);
		Load    : @TEditWindow.Load;
		Store   : @TEditWindow.Store);

	RSFXFileWindow : TStreamRec = (
		ObjType : 15217;
		VmtLink : Ofs(TypeOf(TSFXFileWindow)^);
		Load    : @TSFXFileWindow.Load;
		Store   : @TSFXFileWindow.Store);

implementation

{********** TSFXEditWindow **********}

constructor TSFXEditWindow.Init(AParent: PWindowsObject; ATitle: PChar);
begin
	TEditWindow.Init(AParent, ATitle);
	if IsFlagSet(wb_MDIChild) then
		DefaultProc := @SFXDefMDIChildProc
	else
		DefaultProc := @SFXDefWindowProc;
	with Attr do
		Style := Style or mws_3DFrame;
	with Editor^.Attr do
		Style := Style  and (not ws_Border);
end;

constructor TSFXEditWindow.Load(var S: TStream);
begin
	TEditWindow.Load(S);
	S.Read(DefaultProc, SizeOf(DefaultProc));
end;

procedure TSFXEditWindow.Store(var S: TStream);
begin
	TEditWindow.Store(S);
	S.Write(DefaultProc, SizeOf(DefaultProc));
end;

procedure TSFXEditWindow.GetWindowClass (var AWndClass: TWndClass);
begin
	TEditWindow.GetWindowClass(AWndClass);
	AWndClass.hbrBackground := GetStockObject(Null_Brush);
end;

procedure TSFXEditWindow.SetUpWindow;
begin
   TEditWindow.SetUpWindow;
   MakeDefaultFont(TheFont);
   SendMessage(Editor^.hWindow, wm_SetFont, TheFont, 1);
end;

procedure TSFXEditWindow.MakeDefaultFont(var AFont: HFont);
begin
  FillChar(Font, SizeOf(TLogFont), #0);
  with Font do
  begin
    lfHeight        := 17;     {Make a small font                 }
    lfWeight        := 100;    {Indicate a Bold attribute         }
    lfItalic        := 0;      {Non-zero value indicates italic   }
    lfUnderline     := 0;      {Non-zero value indicates underline}
    lfOutPrecision  := Out_Stroke_Precis;
    lfClipPrecision := Clip_Stroke_Precis;
    lfQuality       := Proof_Quality;
    lfPitchAndFamily:= Variable_Pitch;
    StrCopy(lfFaceName, 'Arial');
  end;
  TheFont := CreateFontIndirect(Font);
end;

procedure TSFXEditWindow.WMCtlColor (var Msg: TMessage);
begin
	if (Attr.Style and mws_SFXFrame <> 0) and (Msg.lParamHi =	CtlColor_Edit) then
	begin
		Msg.Result := GetStockObject(LtGray_Brush);
		SetBkColor(Msg.wParam, RGB(192, 192, 192));
		SetBkMode(Msg.wParam, Opaque);
	end
	else
		DefWndProc(Msg);
end;

procedure TSFXEditWindow.WMSize (var Msg: TMessage);
var
	i : Integer;
	lStyle : LongInt;
begin
	TWindow.WMSize(Msg);
	i := 1;
	if Attr.Style and mws_SFXFrame <> 0 then
	begin
		if IsZoomed(HWindow) and IsFlagSet(wb_MDIChild) then
		begin
			lStyle := GetWindowLong(Parent^.HWindow, gwl_Style);
			if lStyle and mws_SFXFrame <> 0 then
				i := 0;
		end
		else
			i := 0;
	end;
	SetWindowPos(Editor^.HWindow, 0, -i, -i, Msg.LParamLo + i*2, Msg.LParamHi + i*2, swp_NoZOrder);
end;

{********** TSFXFileWindow **********}

constructor TSFXFileWindow.Init (AParent: PWindowsObject; ATitle, AFileName: PChar);
begin
	TFileWindow.Init(AParent, ATitle, AFileName);
	if IsFlagSet(wb_MDIChild) then
		DefaultProc := @SFXDefMDIChildProc
	else
		DefaultProc := @SFXDefWindowProc;
	with Attr do
		Style := Style or mws_3DFrame;
	with Editor^.Attr do
		Style := Style  and (not ws_Border);
end;

constructor TSFXFileWindow.Load(var S: TStream);
begin
	TFileWindow.Load(S);
	S.Read(DefaultProc, SizeOf(DefaultProc));
end;

procedure TSFXFileWindow.Store(var S: TStream);
begin
	TFileWindow.Store(S);
	S.Write(DefaultProc, SizeOf(DefaultProc));
end;

procedure TSFXFileWindow.GetWindowClass (var AWndClass: TWndClass);
begin
	TFileWindow.GetWindowClass(AWndClass);
	AWndClass.hbrBackground := GetStockObject(Null_Brush);
end;

procedure TSFXFileWindow.SetUpWindow;
begin
   TFileWindow.SetUpWindow;
   MakeDefaultFont(TheFont);
   SendMessage(Editor^.hWindow, wm_SetFont, TheFont, 1);
end;

procedure TSFXFileWindow.MakeDefaultFont(var AFont: HFont);
begin
  FillChar(Font, SizeOf(TLogFont), #0);
  with Font do
  begin
    lfHeight        := 17;     {Make a small font                 }
    lfWeight        := 100;    {Indicate a Bold attribute         }
    lfItalic        := 0;      {Non-zero value indicates italic   }
    lfUnderline     := 0;      {Non-zero value indicates underline}
    lfOutPrecision  := Out_Stroke_Precis;
    lfClipPrecision := Clip_Stroke_Precis;
    lfQuality       := Proof_Quality;
    lfPitchAndFamily:= Variable_Pitch;
    StrCopy(lfFaceName, 'Arial');
  end;
  TheFont := CreateFontIndirect(Font);
end;

{ Saves the contents of the TEdit child control into the file currently
  being editted.  Returns true if the file was saved. }
function TSFXFileWindow.Save: Boolean;
begin
  Save := True;
  if Editor^.IsModified then
    if IsNewFile then Save := SaveAs
    else Write;
end;

{ Saves the contents of the TEdit child control into a file whose name
  is retrieved from the user, through execution of a "Save" file
  dialog.  Returns true if the file was saved. }
function TSFXFileWindow.SaveAs: Boolean;
const
DefExt = '*';
szFilterSpec : pchar =
'pascal files (*.pas)'#0'*.pas'#0+
'include files (*.inc)'#0'*.inc'#0+
'ini files (*.ini)'#0'*.ini'#0+
'bat files (*.bat)'#0'*.bat'#0+
'sys files (*.sys)'#0'*.sys'#0+
'text files (*.txt)'#0'*.txt'#0+
'all files (*.*)'#0'*.*'#0;
var
OpenFN       : TOpenFileName;
FullFileName : TFilename;
WinDir       : array [0..145] of Char;
TmpName: array[0..fsPathName] of Char;
begin
  SaveAs := False;
  if FileName <> nil then StrCopy(TmpName, FileName)
  else TmpName[0] := #0;
  GetWindowsDirectory(WinDir, SizeOf(WinDir));
  SetCurDir(WinDir);
  StrCopy(FullFileName, '');
  FillChar(OpenFN, SizeOf(TOpenFileName), #0);
  with OpenFN do
  begin
    hInstance      := GetSFXInstance;
    hwndOwner      := HWindow;
    lpstrDefExt    := DefExt;
    lpstrFile      := FullFileName;
    lpstrFilter    := szFilterSpec ;
    lpstrFileTitle := FileName;
    lpstrTitle     := 'File SaveAs';
    flags          := ofn_OverwritePrompt;
    lStructSize    := sizeof(TOpenFileName);
    nFilterIndex   := 1;       {Index into Filter String in lpstrFilter}
    nMaxFile       := SizeOf(FullFileName);
    lpTemplateName := GetSFXTemplate('SFX3DDlg', DLG_OPENSAVE);
    if (lpTemplateName <> nil)
    then
       flags := flags or ofn_EnableTemplate
    else
       lpTemplateName := nil;
    end;
  if GetSaveFileName(OpenFN) then
 {do something}
 begin
       with OpenFN do
       begin
         StrCopy(TmpName, lpstrFile);  { Save selected file name }
         SetFileName(TmpName);
         Write;
         SaveAs := True;
       end;
    end;
end;

{ Responds to an incoming "Save" command (with a cm_FileSave command
  identifier) by calling Self.Save. }
procedure TSFXFileWindow.CMFileSave(var Msg: TMessage);
begin
  Save;
end;

{ Responds to an incoming "SaveAs" command (with a cm_FileSaveAs command
  identifier) by calling Self.SaveAs. }
procedure TSFXFileWindow.CMFileSaveAs(var Msg: TMessage);
begin
  SaveAs;
end;

procedure TSFXFileWindow.WMCtlColor (var Msg: TMessage);
begin
	if (Attr.Style and mws_SFXFrame <> 0) and (Msg.lParamHi =	CtlColor_Edit) then
	begin
		Msg.Result := GetStockObject(LtGray_Brush);
		SetBkColor(Msg.wParam, RGB(192, 192, 192));
		SetBkMode(Msg.wParam, Opaque);
	end
	else
		DefWndProc(Msg);
end;

procedure TSFXFileWindow.WMSize (var Msg: TMessage);
var
	i : Integer;
	lStyle : LongInt;
begin
	TWindow.WMSize(Msg);
	i := 1;
	if Attr.Style and mws_SFXFrame <> 0 then
	begin
		if IsZoomed(HWindow) and IsFlagSet(wb_MDIChild) then
		begin
			lStyle := GetWindowLong(Parent^.HWindow, gwl_Style);
			if lStyle and mws_SFXFrame <> 0 then
				i := 0;
		end
		else
			i := 0;
	end;
	SetWindowPos(Editor^.HWindow, 0, -i, -i, Msg.LParamLo + i*2, Msg.LParamHi + i*2, swp_NoZOrder);
end;

{procedure TSFXFileWindow.WMWindowPosChanging (var Msg: TMessage);
var
	WinPos : PWindowPos;
	Rect   : TRect;
begin
	LongInt(WinPos) := Msg.lParam;
	(* GetParent returns the real parent of HWindow, the MDI client window.
	 * Parent^.HWindow is the main window. Using the MDI client window allows
	 * alows for the space taken up by the toolbar.
	 *)
	GetWindowRect(GetParent(HWindow), Rect);
	with WinPos^ do
	begin
		if x < 0 then
			x := 0
		else
		if x > (Rect.Right - Rect.Left - cx) then
			x := Rect.Right - Rect.Left - cx;
		if y < 0 then
			y := 0
		else
		if y > (Rect.Bottom - Rect.Top - cy) then
			y := Rect.Bottom - Rect.Top - cy;
		DefMDIChildProc(HWindow, Msg.Message, Msg.wParam, LongInt(WinPos));
	end;
end;}

{********** RegisterSFXWnds **********}

procedure RegisterSFXWnds;
begin
	RegisterType(RSFXEditWindow);
	RegisterType(RSFXFileWindow);
end;

begin
	RegisterSFXWnds;
end.
