'=========================================
'VB/Rig 1.0 - Copyright PC HELP-LINE, 1994
'=========================================

'VBRIGSTD.BAS - Standard Edition

'This code module must be included in every VB
'project that has been rigged for error trapping.
'It contains the CALLed procedures that support the
'error trapping subsystem.

'You are NOT licensed to re-distribute this module
'as source code, either in its original form or in
'modified form.


Option Explicit

' Public symbols

Global Const VBRIG_SHOW_ERROR = 1
Global Const VBRIG_PUSH_PROC_STACK = 2
Global Const VBRIG_POP_PROC_STACK = 3
Global Const VBRIG_FIXUP_PROC_STACK = 4
Global Const VBRIG_GET_PROC_STACK = 5

Global Const VBRIG_PROC_ERRORS_TRAPPED = "+"
Global Const VBRIG_PROC_ERRORS_NOT_TRAPPED = "-"

' Private symbols

Const VBRIG_TITLE = "VB/Rig"
Const VBRIG_INI_SECTION = "VB/Rig"
Const VBRIG_STACK_INCREMENT = 8


' ======================
' Visual Basic constants
' ======================

' Show parameters
Const MODAL = 1

' Mouse pointers
Const DEFAULT = 0

' MsgBox parameters
Const MB_ICONEXCLAMATION = 48


' =====================
' Windows API functions
' =====================
Const WM_USER = 1024
Const LB_SETTABSTOPS = WM_USER + 19
Const LB_SETHORIZONTALEXTENT = WM_USER + 21
Const EM_SETTABSTOPS = WM_USER + 27

Declare Function vbrig_nGetProfileString Lib "Kernel" Alias "GetProfileString" (ByVal lpAppName As String, ByVal lpKeyName As String, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Integer) As Integer
Declare Function vbrig_nlSetTabstops Lib "User" Alias "SendMessage" (ByVal hWnd As Integer, ByVal wMsg As Integer, ByVal wParam As Integer, lParam As Any) As Long
Declare Function vbrig_nlSetHorizScrollBar Lib "User" Alias "SendMessage" (ByVal hWnd As Integer, ByVal wMsg As Integer, ByVal wParam As Integer, lParam As Any) As Long

Private Sub priVBRig_InternalError (nRuntimeError As Integer)

'This procedure is invoked if a runtime error occurs within
'the VBRIG.BAS module.  Theoretically, this should never
'actually happen.  If it does, then there's a bug in the
'VBRIG code that needs to be fixed.  Please contact
'PC HELP-LINE if a VBRIG "internal error" occurs in your
'application.


Beep
MsgBox "Error" & Str$(nRuntimeError) & " - " & Error$(nRuntimeError), MB_ICONEXCLAMATION, "- " & VBRIG_TITLE & " (internal) -"

End Sub

Private Sub priVBRig_MonitorStack (nProcStackPointer As Integer, sProcStack() As String)

'This procedure allows you to monitor the state of the
'procedure stack in the debug window, while your application
'is running.  To enable this feature, simply uncomment the
'following VB statement.

'Debug.Print Right$(Space$(2) & Str$(nProcStackPointer), 2); Space$(nProcStackPointer * 2); sProcStack(nProcStackPointer); Space$(2); Format$(Now, "hh:mm:ss")

End Sub

Private Static Sub priVBRig_ProcStack (nVBRig_Command As Integer, sModuleIdentifier As String, sProcedureName As String, sReturnStack() As String)

'This procedure provides the routines used to maintain
'your application's "procedure stack".

Dim sProcStack() As String, sProcName As String, sErrMsg As String
Dim nProcStackPointer As Integer, nSub As Integer


On Error GoTo priVBRig_ProcStack_Err

Select Case nVBRig_Command
   Case VBRIG_PUSH_PROC_STACK
      If nProcStackPointer = 0 Then
         ReDim sProcStack(VBRIG_STACK_INCREMENT)
      End If
      
      nProcStackPointer = nProcStackPointer + 1

      If nProcStackPointer > UBound(sProcStack) Then
         ReDim Preserve sProcStack(UBound(sProcStack) + VBRIG_STACK_INCREMENT)
      End If
      
      sProcStack(nProcStackPointer) = priVBRig_sFormatStack(sModuleIdentifier, sProcedureName)

      Call priVBRig_MonitorStack(nProcStackPointer, sProcStack())

   Case VBRIG_POP_PROC_STACK
      sProcName = priVBRig_sFormatStack(sModuleIdentifier, sProcedureName)
      
      If nProcStackPointer >= 1 Then
         If sProcName = sProcStack(nProcStackPointer) Then
            Call priVBRig_MonitorStack(nProcStackPointer, sProcStack())

            nProcStackPointer = nProcStackPointer - 1
         Else
            sErrMsg = "VB/Rig's procedure stack is out of sync.  Check the " & Chr$(34) & "Last Push" & Chr$(34) & " "
            sErrMsg = sErrMsg & "procedure for " & Chr$(34) & "Exit" & Chr$(34) & " statements that have not been "
            sErrMsg = sErrMsg & "properly rigged." & Chr$(10) & Chr$(10)
            sErrMsg = sErrMsg & "Last Push: " & sProcStack(nProcStackPointer) & Chr$(10)
            sErrMsg = sErrMsg & "This Pop: " & sProcName & Chr$(10) & Chr$(10)
            sErrMsg = sErrMsg & "A re-sync will be attempted."
            
            Beep
            MsgBox sErrMsg, MB_ICONEXCLAMATION, "- " & VBRIG_TITLE & " (internal) -"
            
            For nSub = nProcStackPointer To 1 Step -1
               If sProcStack(nSub) = sProcName Then
                  Call priVBRig_MonitorStack(nSub, sProcStack())
                  
                  nProcStackPointer = nSub - 1
                  ReDim Preserve sProcStack(nProcStackPointer)
                  Exit For
               End If
            Next
         End If
      Else
         nProcStackPointer = 0
      End If

   Case VBRIG_FIXUP_PROC_STACK
      sProcName = priVBRig_sFormatStack(sModuleIdentifier, sProcedureName)
      
      For nSub = nProcStackPointer To 1 Step -1
         If sProcStack(nSub) = sProcName Then
            nProcStackPointer = nSub
            ReDim Preserve sProcStack(nProcStackPointer)
            Exit For
         End If
      Next

   Case VBRIG_GET_PROC_STACK
      ReDim sReturnStack(nProcStackPointer)
      sProcName = ""

      For nSub = 1 To nProcStackPointer
         If nSub > 1 Then
            sProcName = sProcName & ","
         End If

         sProcName = sProcName & sProcStack(nSub)
         sReturnStack(nSub) = sProcStack(nSub)
      Next

      sProcedureName = sProcName

   Case Else
End Select

Exit Sub


'======================
priVBRig_ProcStack_Err:
'======================
Call priVBRig_InternalError(Err)
Resume Next

End Sub

Private Function priVBRig_sFormatStack (sModuleIdentifier As String, sProcedureName As String) As String

'This procedure arranges the module name and procedure name
'in a consistent format, for placement on the stack.

priVBRig_sFormatStack = Left$(sProcedureName, Len(VBRIG_PROC_ERRORS_TRAPPED)) & sModuleIdentifier & "(" & Mid$(sProcedureName, Len(VBRIG_PROC_ERRORS_TRAPPED) + 1) & ")"

End Function

Sub VBRig_Error (nVBRig_Command As Integer, nRuntimeError As Integer, sCustomErrorMessage As String, sModuleIdentifier As String, sProcedureName As String)

'This procedure is the central "entry point" for all the
'VBRIG functions.  It is the only routine in VBRIG that
'is directly accessible by your application.

Dim nHandleAppend As Integer, nSub As Integer, nPos As Integer
Dim nStackErrorOrigin As Integer, nStackErrorTrap As Integer
Dim nKeyValueLength As Integer, nTabstop() As Integer
Dim tfSelected As Integer, nMousePointerSave As Integer
Dim nlRC As Long
Dim sProcStack() As String, sProcStackString As String
Dim sProcTrap As String, sShowTrapEnabled As String, sShowProc As String
Dim sShowTrap As String, sShowOrigin As String
Dim sErrMsg As String
Dim sKeyValue As String


On Error GoTo VBRig_Error_Err

If Len(sCustomErrorMessage) = 0 Then
   sErrMsg = Error$(nRuntimeError)

   If Right$(sErrMsg, 1) <> "." Then
      sErrMsg = sErrMsg & "."
   End If
Else
   sErrMsg = sCustomErrorMessage
End If

Select Case nVBRig_Command
   Case VBRIG_SHOW_ERROR
      Load frmVBRig
      frmVBRig.Move (Screen.Width - frmVBRig.Width) / 2, (Screen.Height - frmVBRig.Height) / 2
      
      frmVBRig.Caption = "- " & VBRIG_TITLE & " -"
      frmVBRig.lblErr.Caption = LTrim$(Str$(nRuntimeError))
      frmVBRig.lblErrMsg.Caption = sErrMsg


      Call priVBRig_ProcStack(VBRIG_GET_PROC_STACK, "", "", sProcStack())
      GoSub VBRig_Error_GetErrorOriginAndTrap
      GoSub VBRig_Error_DisplayProcStack

      nMousePointerSave = Screen.MousePointer
      Screen.MousePointer = DEFAULT

      frmVBRig.Show MODAL
      Unload frmVBRig

      Screen.MousePointer = nMousePointerSave

   Case Else
End Select

Select Case nVBRig_Command
   Case VBRIG_SHOW_ERROR
      Call priVBRig_ProcStack(VBRIG_FIXUP_PROC_STACK, sModuleIdentifier, sProcedureName, sProcStack())

   Case VBRIG_PUSH_PROC_STACK, VBRIG_POP_PROC_STACK, VBRIG_FIXUP_PROC_STACK
      Call priVBRig_ProcStack(nVBRig_Command, sModuleIdentifier, sProcedureName, sProcStack())

   Case Else
End Select

Exit Sub


'=================================
VBRig_Error_GetErrorOriginAndTrap:
'=================================
nStackErrorOrigin = UBound(sProcStack)
nStackErrorTrap = 0

sProcTrap = priVBRig_sFormatStack(sModuleIdentifier, sProcedureName)

For nSub = UBound(sProcStack) To 1 Step -1
   If sProcStack(nSub) = sProcTrap Then
      nStackErrorTrap = nSub
      Exit For
   End If
Next

Return


'============================
VBRig_Error_DisplayProcStack:
'============================
frmVBRig.lstStack.Clear

For nSub = 1 To UBound(sProcStack)
   sShowTrapEnabled = Left$(sProcStack(nSub), Len(VBRIG_PROC_ERRORS_TRAPPED))
   sShowProc = Mid$(sProcStack(nSub), Len(VBRIG_PROC_ERRORS_TRAPPED) + 1)

   tfSelected = False

   If sShowTrapEnabled = VBRIG_PROC_ERRORS_NOT_TRAPPED Then
      sShowTrapEnabled = Space$(1)
   End If

   If nSub = nStackErrorTrap Then
      sShowTrap = ">>"
      tfSelected = True
   Else
      sShowTrap = Space$(1)
   End If

   If nSub = nStackErrorOrigin Then
      sShowOrigin = ">>"
      tfSelected = True
   Else
      sShowOrigin = Space$(1)
   End If

   frmVBRig.lstStack.AddItem sShowOrigin & Chr$(9) & sShowTrap & Chr$(9) & sShowTrapEnabled & Chr$(9) & sShowProc
   frmVBRig.lstStack.ItemData(frmVBRig.lstStack.NewIndex) = tfSelected
Next

If nStackErrorTrap > 0 Then
   frmVBRig.lstStack.TopIndex = nStackErrorTrap - 1
End If
If frmVBRig.lstStack.ListCount > 0 Then
   frmVBRig.lstStack.Selected(0) = True  'trigger .Selected refresh
End If


'format listbox columns and headings
frmVBRig.txtStack.Text = "Origin" & Chr$(9) & "Trap" & Chr$(9) & Space$(1) & Chr$(9) & "Procedure"

ReDim nTabstop(2)
nTabstop(0) = 22
nTabstop(1) = 42
nTabstop(2) = 50

nlRC = vbrig_nlSetTabstops(frmVBRig.txtStack.hWnd, EM_SETTABSTOPS, UBound(nTabstop) + 1, nTabstop(0))
nlRC = vbrig_nlSetTabstops(frmVBRig.lstStack.hWnd, LB_SETTABSTOPS, UBound(nTabstop) + 1, nTabstop(0))
nlRC = vbrig_nlSetHorizScrollBar(frmVBRig.lstStack.hWnd, LB_SETHORIZONTALEXTENT, (frmVBRig.lstStack.Width * 2) \ Screen.TwipsPerPixelX, 0)

Return


'===============
VBRig_Error_Err:
'===============
Call priVBRig_InternalError(Err)
Resume Next

End Sub

